@@ -1,7 +1,7 @@
-from django.shortcuts import render
 from django_tables2 import SingleTableMixin
 from django.shortcuts import redirect
 from django.http import HttpResponseRedirect
+from django.urls import reverse_lazy
 from pandas import isna, DataFrame as DF, to_datetime
 import inspect
 from django.contrib import messages
@@ -12,22 +12,26 @@ import plotly.express as px
 from plotly import offline
 
 
+
 class SaveFilterMixin(SingleTableMixin):
     """ This Mixin Can be used with a FilterView SingleTable in order to save
     the users filter selections after navegating away from the lookup page"""
 
     def get(self, request, *args, **kwargs):
         view_name = self.__class__
-        qstr = self.request.GET.urlencode()
+        qstr = request.GET.urlencode()
         full_path = request.get_full_path()
-
         if qstr != '':
-            self.request.session[f'{view_name}_fp'] = full_path
+            self.request.session[f'{view_name}_qstr'] = qstr
             return super().get(request)
         else:
-            fp = self.request.session.get(f'{view_name}_fp')
-            if isna(fp) or '_export' in fp:
-                return super().get(request)
+            qstr = request.session.get(f'{view_name}_qstr')
+            if qstr != None:
+                fp = full_path + '?' + qstr
+            else:
+                fp = full_path
+            if isna(qstr) or '_export' in fp:
+                return super().get(request, *args, **kwargs)
             else:
                 return redirect(fp)
 
@@ -39,54 +43,69 @@ class SaveFilterMixin(SingleTableMixin):
             return kwargs
 
 class InlineFormsetMixin:
-    formset_helper = None
+    factories = [] # list of dictionaries that must contain the key factory and the value of a formset factory instance, helper and herder are optional
     form_helper = None
+    template_name = 'oee/inline-formset.html'
+    addlines_url = None # set this in the view to have the addlines btn redirect here with self.object.pk as an arg
     
-    def get_context_data(self,*args,**kwargs):
-        context=super().get_context_data(*args,**kwargs)
-        if self.request.POST:
-            context['formset'] = self.formset(self.request.POST,instance=self.object)
-        else:
-            context['formset'] = self.formset(instance=self.object)
-        context['formset_helper']=self.formset_helper if self.formset_helper else None
+    def get_context_data(self, *args, factories=None, **kwargs):
+        context = super().get_context_data(*args, **kwargs)
+        if factories == None:
+            factories = [fd.copy() for fd in self.factories]
+            for fd in factories:
+                if self.request.POST:
+                    fd['factory'] = fd['factory'](self.request.POST, instance=self.object)
+                else:
+                    fd['factory'] = fd['factory'](instance=self.object)
+        context['factories'] = factories
         context['form_helper']=self.form_helper if self.form_helper else None
-        return context               
-        
+        return context
+            
     def post(self, request, *args, **kwargs):
-        next = self.request.session.get('next')
-        cancel = 'cancel' in self.request.POST.keys()
-        print(cancel,next)
-        if next and cancel:
-            return HttpResponseRedirect(next)
         try:
             self.object = self.get_object()
         except AttributeError:
             self.object = None
         context = self.get_context_data()
         form = self.get_form()
-        child = context['formset']
-        if form.is_valid() and child.is_valid():
+        if not form.is_valid():
+            print(form.errors)
+            return self.form_invalid(form)
+        fdsv = True
+        for fd in context['factories']:
+            if not fd['factory'].is_valid():
+                print(fd['factory'].errors)
+                fdsv = False
+        if fdsv and form.is_valid():
             return self.form_valid(form)
         else:
-            return self.form_invalid(form)
+            return self.form_invalid(form=form, factories=context['factories'])
 
-    def form_valid(self,form):
+    def form_valid(self, form):
         self.object = form.save()
         context = self.get_context_data()
-        children_names = ['formset']
-        for children_name in children_names:
-            child = context[children_name] 
-            child.is_valid()
-            child.save()
+        for fd in context['factories']:
+            if fd['factory'].is_valid():
+                fd['factory'].save()
         self.object.save()
         return super().form_valid(form)
 
+    def form_invalid(self, form, factories):
+        return self.render_to_response(self.get_context_data(form=form, factories=factories))
+
     def get_success_url(self):
-        if 'addlines' in self.request.POST:
+        if 'addlines' in self.request.POST and self.addlines_url:
+            #user defined a redirect url for the addlines btn, default behavior sends pk as arg
+            return reverse_lazy(self.addlines_url, kwargs={'pk':self.object.pk})
+        elif 'addlines' in self.request.POST and not self.addlines_url:
+            print('\n\n\nI am here\n\n\n')
+            #adding lines to form but no redirect url given. Use default behavior
             return self.request.path_info
         else:
+            #add lines was not clicked, use default success_url
             return super().get_success_url()
 
+
 class SaveFormMixin:
     """ This Mixin Can be used with any view that uses a form mixin to
     save a users form selection even if they navigate away from the page"""
@@ -150,9 +169,9 @@ class RedirectPrevMixin:
 
     def get_success_url(self):
         ''' If next was stored redirect there, otherwise return super() '''
+        print('\n\n\nI am REdirect prev\n\n\n')
         next = self.request.session.get('next')
-        m = 'addlines' not in self.request.POST # 
-        if next and m: 
+        if next: 
             return next
         return super().get_success_url()
 
@@ -173,9 +192,10 @@ class CheckGroupPermMixin(UserPassesTestMixin):
     allow_superusers = True #If true then a django superuser will be granted access regardless of group membership
 
     def test_func(self):
-        ''' an extended test_func that checks group membership '''
-        #First the parent class' test_func must pass
-        if super().test_func() == False: return False
+        ''' an overwritten UserPassesTest test_func that checks group membership '''
+        #if allowed groups is an empty list it is taken to be permissive
+        if self.allowed_groups == []: return True
+        #Now begin the test to see if this user can proceed
         #Test 1: Is the user a super user? (If allowed)
         if self.allow_superusers and self.request.user.is_superuser:
             return True
@@ -185,7 +205,6 @@ class CheckGroupPermMixin(UserPassesTestMixin):
         if len(intersect) > 0:
             return True
         return False
-    
 
 class SinglePlotMixin:
     ''' This mixin creates a potly figure based on a FilterView that is using
